package org.processmining.plugins;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.packagemanager.PMController;
import org.processmining.contexts.uitopia.packagemanager.PMPackage;
import org.processmining.framework.boot.Boot;
import org.processmining.framework.util.HTMLToString;

//@Plugin(name = "Show Runner-up Packages", parameterLabels = {}, returnLabels = { "Release info" }, returnTypes = { HTMLToString.class }, userAccessible = true)
public class RunnerUpPackagesPlugin implements HTMLToString {

	private Set<String> allEPrimaryPacks;
	private Set<String> allESecondaryPacks;
	private Set<String> allPrimaryPacks;
	private Set<String> allSecondaryPacks;
	private List<? extends PMPackage> packages;

//	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
//	@PluginVariant(variantLabel = "Default", requiredParameterLabels = {})
	public static HTMLToString info(final UIPluginContext context) {
		return new RunnerUpPackagesPlugin();
	}

	public void addAll(Set<String> primary, Set<String> secondary) {
		PMController packageController = new PMController(Boot.Level.NONE);
		packages = packageController.getToUninstallPackages();
		Map<String, PMPackage> map = new HashMap<String, PMPackage>();
		Set<String> toDo;
		for (PMPackage pack : packages) {
			map.put(pack.getPackageName(), pack);
		}

		primary.add("ComparisonFramework"); primary.add("PartialOrderReplayer"); primary.add("InductiveVisualMiner"); 

		toDo = new HashSet<String>(primary);
		secondary.addAll(primary);
		while (!toDo.isEmpty()) {
			String name = toDo.iterator().next();
			toDo.remove(name);
			secondary.add(name);
			if (!map.containsKey(name)) {
				System.err.println("Unknown runner-up package: " + name);
			} else {
				Set<String> deps = new HashSet<String>(map.get(name).getDependencies());
				deps.removeAll(secondary);
				toDo.addAll(deps);
			}
		}
		secondary.removeAll(primary);
	}

	public RunnerUpPackagesPlugin() {
		EstablishedPackagesPlugin established = new EstablishedPackagesPlugin();
		allEPrimaryPacks = new HashSet<String>();
		allESecondaryPacks = new HashSet<String>();
		allPrimaryPacks = new HashSet<String>();
		allSecondaryPacks = new HashSet<String>();
		established.addAll(allEPrimaryPacks, allESecondaryPacks);
		addAll(allPrimaryPacks, allSecondaryPacks);
		allPrimaryPacks.removeAll(allEPrimaryPacks);
		allPrimaryPacks.removeAll(allESecondaryPacks);
		allSecondaryPacks.removeAll(allEPrimaryPacks);
		allSecondaryPacks.removeAll(allESecondaryPacks);
	}

	public String toHTMLString(boolean includeHTMLTags) {
		StringBuffer buffer = new StringBuffer();

		if (includeHTMLTags) {
			buffer.append("<html>");
		}

		buffer.append("<h1>Runner-up Packages</h1>");
		
		toHTMLString("Primary Packages", allPrimaryPacks, buffer);
		toHTMLString("Secondary Packages", allSecondaryPacks, buffer);

		if (includeHTMLTags) {
			buffer.append("</html>");
		}
		return buffer.toString();
	}

	private void toHTMLString(String header, Set<String> names, StringBuffer buffer) {
		buffer.append("<h2>" + header + "</h2>");
		buffer.append("<table>");
		buffer.append("<tr><th>Package</th><th>Version</th><th>Author</th></tr>");
		for (PMPackage pack : packages) {
			if (names.contains(pack.getPackageName())) {
				buffer.append("<tr>");
				buffer.append("<td>" + pack.getPackageName() + "</td>");
				buffer.append("<td>" + pack.getVersion() + "</td>");
				buffer.append("<td>" + pack.getAuthorName() + "</td>");
				buffer.append("</tr>");
			}
		}
		buffer.append("</tr>");
		buffer.append("</table>");
	}
}
